# -*- coding: utf-8 -*-
# PyBEST: Pythonic Black-box Electronic Structure Tool
# Copyright (C) 2016-- The PyBEST Development Team
#
# This file is part of PyBEST.
#
# PyBEST is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# PyBEST is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>
#
# --
#
# pylint: skip-file


import pybest.gbasis.gobasis as pybest_basis
from pybest.context import context
from pybest.gbasis.gobasis import (
    compute_eri,
    compute_cholesky_eri,
    compute_kinetic,
    compute_nuclear,
    compute_nuclear_repulsion,
    compute_overlap,
    get_gobasis,
)
from pybest.linalg import DenseLinalgFactory, CholeskyLinalgFactory


def prepare_mol(filen, nbasis, linalg_set, orb=None):

    fn = context.get_fn(f"test/{filen}")
    obasis = pybest_basis.get_gobasis(nbasis, fn, print_basis=False)
    lf = linalg_set(obasis.nbasis)
    olp = pybest_basis.compute_overlap(obasis)
    kin = pybest_basis.compute_kinetic(obasis)
    na = pybest_basis.compute_nuclear(obasis)
    external = pybest_basis.compute_nuclear_repulsion(obasis)

    if isinstance(lf, CholeskyLinalgFactory):
        er = pybest_basis.compute_cholesky_eri(obasis, threshold=1e-8)
    elif isinstance(lf, DenseLinalgFactory):
        er = pybest_basis.compute_eri(obasis)

    return kin, na, er, external, olp, lf
