# -*- coding: utf-8 -*-
# PyBEST: Pythonic Black-box Electronic Structure Tool
# Copyright (C) 2016-- The PyBEST Development Team
#
# This file is part of PyBEST.
#
# PyBEST is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# PyBEST is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>
#
# --
# pylint: skip-file


import numpy as np
import pytest

import pybest.gbasis.gobasis as pybest_basis
from pybest.wrappers import RHF
from pybest.ci.rci_base import RCI
from pybest.ci.rcis import RCIS
from pybest.ci.test.common import prepare_mol
from pybest.linalg.dense import DenseLinalgFactory
from pybest.scf import AufbauOccModel


linalg_set = [DenseLinalgFactory]
# Reference energies
# HF orbitals
e_ex_cis_hf_ref = [
    0.00000000,
    0.33713109,
    0.40279938,
    0.42397873,
    0.48927984,
    0.56353471,
    0.68179610,
    0.85905279,
    0.89799226,
    0.97464748,
]

e_ex_cis_hf_ref_fc = [
    0.00000000,
    0.33713786,
    0.40280167,
    0.42398123,
    0.48930023,
    0.56354090,
    0.68179914,
    0.85905401,
    0.89800188,
    0.97467242,
]

parameters = [
    (10, 0, e_ex_cis_hf_ref),
    (10, 1, e_ex_cis_hf_ref_fc),
]


@pytest.mark.parametrize("nroot, ncore, exp_result", parameters)
@pytest.mark.parametrize("linalg_set", linalg_set)
def test_cis(nroot, ncore, exp_result, linalg_set):

    kin, na, er, external, olp, lf = prepare_mol(
        "water.xyz", "cc-pvdz", linalg_set
    )

    orb_a = lf.create_orbital()
    occ_model = AufbauOccModel(5)

    hf = RHF(lf, occ_model)
    hf_output = hf(kin, na, er, external, olp, orb_a)

    rcis = RCIS(lf, occ_model, ncore=ncore)
    rcis_output = rcis(kin, na, er, hf_output, nroot=nroot)

    for i in range(nroot):
        assert abs(rcis_output.e_ci[i] - exp_result[i]) < 1e-6
