#!/usr/bin/env python

from pybest import context
from pybest.gbasis import (
    get_gobasis,
    compute_nuclear_repulsion,
    compute_overlap,
    compute_dkh,
    compute_cholesky_eri,
)
from pybest.linalg import DenseLinalgFactory

# DKH2 Hamiltonian for the U dimer (the lowest closed-shell state)
# --------------------------------

# Define coordinate file
# ----------------------
coord = context.get_fn("test/u2.xyz")

# Create a Gaussian basis set
# ---------------------------
gobasis = get_gobasis("ano-rcc-vdz", coord)

# Create a linalg factory
# -----------------------
lf = DenseLinalgFactory(gobasis.nbasis)

# Compute integrals in the atom-centered Gaussian basis set
# ---------------------------------------------------------
dkh = compute_dkh(gobasis)
er = compute_cholesky_eri(gobasis)
nuc = compute_nuclear_repulsion(gobasis)

# Compute overlap matrix of atom-centered basis set
# -------------------------------------------------
olp = compute_overlap(gobasis)

# Create orbitals that store the AO/MO coefficients
# -------------------------------------------------
orb = lf.create_orbital()
